<div class="phileo">
    {% if can_like %}
        <a class="btn ajax"
           data-method="post"
           data-replace-closest=".phileo"
           data-spinner="off"
           href="{{ like_url }}?t=b"
           title="{{ like_text }}">
            <i class="icon {{ like_class }}"></i>
        </a>
    {% endif %}
</div>
